#include <math.h>

#pragma hdrstop

#include "YRTTriangle.h"
#include "YRTUtils.h"

bool TYRTTriangle::GetIntersection(TYRTRay ARay, TYRTVector *APoint, TColor *AColor)
{
	// Get triangle edge vectors and the plane normal:
	TYRTVector u = Points[1] - Points[0];
	TYRTVector v = Points[2] - Points[0];

	TYRTVector norm = u * v;

	if (norm.IsNearZero())
		return false;

	// Note: if the following instruction is moved down, we lose about 5% on speed...
	TYRTVector w0 = ARay.Start - Points[0];

	// Check if the ray is parallel to the triangle plane:
	float b = dot(norm, ARay.Dir);
	if (fabs(b) < EPSILON)
		return false;

	// Check if the ray goes away from the triangle:
	float a = -dot(norm, w0);
	float r = a / b;
	if (r < 0.0)
		return false;

	// The intersection point of the ray with the triangle plane:
	*APoint = ARay.Start + (r * ARay.Dir);

	// Check whether APoint is inside the triangle - get and test parametric coordinates:
	TYRTVector w = *APoint - Points[0];

	float uu = dot(u, u);
	float uv = dot(u, v);
	float vv = dot(v, v);

	float D = uv * uv - uu * vv;

	float wu = dot(w, u);
	float wv = dot(w, v);

	float s = (uv * wv - vv * wu) / D;
	if (s < 0.0 || s > 1.0)
		return false;
	float t = (uv * wu - uu * wv) / D;
	if (t < 0.0 || (s + t) > 1.0)
		return false;

	*AColor = this->Color;
	return true;
}
