#include <math.h>

#pragma hdrstop

#include "YRTShape.h"
#include "YRTUtils.h"
#include "YRTVector.h"

TYRTVector::TYRTVector() : x(0.0f), y(0.0f), z(0.0f)
{
}

TYRTVector::TYRTVector(float a) : x(a), y(a), z(a)
{
}

TYRTVector::TYRTVector(float ax, float ay, float az) : x(ax), y(ay), z(az)
{
}

TYRTVector TYRTVector::operator+ (const TYRTVector& Arg)
{
	return TYRTVector(x + Arg.x, y + Arg.y, z + Arg.z);
}

TYRTVector TYRTVector::operator- (const TYRTVector& Arg)
{
	return TYRTVector(x - Arg.x, y - Arg.y, z - Arg.z);
}

TYRTVector TYRTVector::operator* (const TYRTVector& Arg)
{
	return TYRTVector(y * Arg.z - z * Arg.y, z * Arg.x - x * Arg.z, x * Arg.y - y * Arg.x);
}

float TYRTVector::GetDistance(TYRTVector& Arg)
{
	return sqrt(GetDistanceSqr(Arg));
}

float TYRTVector::GetDistanceSqr(TYRTVector& Arg)
{
	return sqr(x - Arg.x) + sqr(y - Arg.y) + sqr(z - Arg.z);
}

bool TYRTVector::IsNearZero(void)
{
	return fabs(x) < EPSILON && fabs(y) < EPSILON && fabs(z) < EPSILON;
}

bool operator == (const TYRTVector& v1, const TYRTVector& v2)
{
	return v1.x == v2.x && v1.y == v2.y && v1.z == v2.z;  
}

TYRTVector operator* (float s, const TYRTVector& Arg)
{
	return TYRTVector(Arg.x * s, Arg.y * s, Arg.z * s);
}
